/*  xyac - Display YAC infomation in X
 *  
 *  This software is heavily based on :
 *  YAC: Yet Another Caller ID Program - TiVo Listener
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  XYAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  XYAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xosd.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>
#include <sys/time.h>
#include "xyac.h"

static struct option long_options[] = {
  {"font",   1, NULL, 'f'},
  {"color",  1, NULL, 'c'},
  {"colour",  1, NULL, 'c'},
  {"indent",  1, NULL, 'i'},
  {"delay",  1, NULL, 'd'},
  {"offset", 1, NULL, 'o'},
  {"pos",    0, NULL, 'p'},
  {"align",  1, NULL, 'A'},
  {"shadow", 1, NULL, 's'},
  {NULL,     0, NULL, 0}
};

xosd *osd;
int screen_line = 0;

int main (int argc, char *argv[])
{

  char *font = (char*) osd_default_font;
  char *colour = "green";
  int delay = 15;
  xosd_pos pos = XOSD_top;
  int voffset = 0;
  int hoffset = 0;
  int shadow = 0;
  int lines = 1;
  struct timeval old_age;
  xosd_align align=XOSD_right;

  while (1)
    {
      int option_index = 0;
      int c = getopt_long (argc, argv, "l:A:a:f:c:d:o:i:s:p:hw", long_options, &option_index);
      if (c == -1) break;
      switch (c)
	{
	case 'A':
	  if (strcasecmp(optarg,"left")==0) {
	    align=XOSD_left;
	  } else if (strcasecmp(optarg,"right")==0) {
	    align=XOSD_right;
	  } else if (strcasecmp(optarg,"center")==0) {
	    align=XOSD_center;
	  } else if (strcasecmp(optarg,"centre")==0) {
	    align=XOSD_center;
	  } else {
	    fprintf (stderr, "Unknown alignment: %s\n", optarg);
	    return EXIT_FAILURE;
	  }
	  break;
	case 'p':
	  if (strcasecmp(optarg,"top")==0) {
	    pos=XOSD_top;
	  } else if (strcasecmp(optarg,"middle")==0) {
	    pos=XOSD_middle;
	  } else if (strcasecmp(optarg,"bottom")==0) {
	    pos=XOSD_bottom;
	  } else {
	    fprintf (stderr, "Unknown alignment: %s\n", optarg);
	    return EXIT_FAILURE;
	  }
	  break;
	case 'f':
	  font = optarg;
	  break;
	case 'c':
	  colour = optarg;
	  break;
	case 'd':
	  delay = atoi(optarg);
	  break;
	case 'o':
	  voffset = atoi(optarg);
	  break;
	case 'i':
	  hoffset = atoi(optarg);
	  break;
	case 's':
	  shadow=atoi(optarg);
	  break;
	case '?':
	case 'h':
	  fprintf (stderr, "Usage: %s [OPTION] ...\n", argv[0]);
	  fprintf (stderr, "Listen for YAC requests and display them using XOSD\n\n");
	  fprintf (stderr, "  -p, --pos=(top|middle|bottom)\n");
	  fprintf (stderr, "                      Display at top/middle/bottom of screen. Top is default\n");
	  fprintf (stderr, "  -A, --align=(left|right|center)\n");
	  fprintf (stderr, "                      Display at left/right/center of screen.Left is default\n");
	  fprintf (stderr, "  -f, --font=FONT     Use font (default: %s)\n", osd_default_font);
	  fprintf (stderr, "  -c, --colour=COLOUR Use colour\n");
	  fprintf (stderr, "  -d, --delay=TIME    Show for specified time\n");
	  fprintf (stderr, "  -o, --offset=OFFSET Vertical Offset\n");
	  fprintf (stderr, "  -i, --indent=OFFSET Horizontal Offset\n");
	  fprintf (stderr, "  -h, --help          Show this help\n");
	  fprintf (stderr, "  -s, --shadow=SHADOW Offset of shadow, default is 0 which is no shadow\n");
	  return EXIT_SUCCESS;
	}
    }

  osd = xosd_create (lines);
  if (!osd)
    {
      fprintf (stderr, "Error initializing osd: %s\n", xosd_error);
      return EXIT_FAILURE;
    }

  xosd_set_font(osd, font);
  xosd_set_colour(osd, colour);
  xosd_set_timeout(osd, delay);
  xosd_set_pos(osd, pos);
  xosd_set_vertical_offset(osd, voffset);
  xosd_set_horizontal_offset(osd, hoffset);
  xosd_set_shadow_offset(osd, shadow);
  xosd_set_align(osd, align);


  xosd_set_align (osd, align);
  /* Not really needed, but at least we aren't throwing around an unknown value */
  old_age.tv_sec=0;

  doYAC();

  if (xosd_is_onscreen(osd)) {
    xosd_wait_until_no_display(osd);
  }

  xosd_destroy (osd);

  return EXIT_SUCCESS;
}

void DisplayTextOnOSD(char *buffer) {
    xosd_display (osd, screen_line, XOSD_string, buffer);
}
