/*  xyac - Display YAC infomation in X
 *  
 *  This software is heavily based on :
 *  YAC: Yet Another Caller ID Program - TiVo Listener
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  XYAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  XYAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "xyac.h"

// GLOBALS
int     clientFd        = 0;
int     serverFd        = 0;
bool    fDebug          = false;
bool    fQuit           = false;
bool    fListening      = false;
bool    fDisplaying     = false;

void doYAC() {
    int serverLen, clientLen;
    int port = PORT_DEFAULT;
    int on = 1;
    char szInput[MAX_CHARS + 1];
    char szName[MAX_CHARS], szNumber[MAX_CHARS];
    bool fCallerID;
    struct sockaddr_in serverINETAddress;
    struct sockaddr* serverSockAddrPtr;
    struct sockaddr_in clientINETAddress;
    struct sockaddr* clientSockAddrPtr;

    // Exit cleanly if user presses CTRL-C
    (void) signal (SIGINT, QuitProgram);

    // Main program loop
    while (!fQuit)
    {
        if (fDebug)
            printf("Initializing network connection...\n");

        serverFd = socket(AF_INET, SOCK_STREAM, 0);
        serverLen = sizeof(serverINETAddress);
        setsockopt(serverFd, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on));

        bzero((char*) &serverINETAddress, serverLen);
        serverINETAddress.sin_family = AF_INET;
        serverINETAddress.sin_addr.s_addr = htonl(INADDR_ANY);
        serverINETAddress.sin_port = htons(port);

        clientLen = sizeof(clientINETAddress);
        bzero((char*) &clientINETAddress, clientLen);

        serverSockAddrPtr = (struct sockaddr*) &serverINETAddress;
        bind(serverFd, serverSockAddrPtr, serverLen);

        if (fDebug)
            printf("Listening on port %d...\n", port);

        fListening = true;
        listen(serverFd, 1);

        clientSockAddrPtr = (struct sockaddr*) &clientINETAddress;
        clientFd = accept(serverFd, clientSockAddrPtr, &clientLen);

        fListening = false;

        if (fDebug)
            printf("Client connected.  Reading data from connected socket...\n");

        bzero(szInput, sizeof(szInput));
        ReadLineFromSocket(clientFd, szInput);

        if (fDebug)
            printf("Closing network connection to client.\n");

        close(clientFd);
        clientFd = 0;

        if (szInput[0] == '\0')
        {
            if (fDebug)
                printf("Client sent no data, no need to show anything on the screen.\n");
        }
        else
        {
            // Need to parse the data to see if it's Caller ID or not.
            fCallerID = DecodeCallerIDInfo(szInput, szName, szNumber);

            if (fCallerID)
            {
                if (fDebug)
                    printf("The received data was Caller ID information.\n");

                sprintf(szInput, "%s  %s", szName, szNumber);
            }

            if (fDebug)
                printf("Displaying on Screen: \n%s\n", szInput);

            DisplayTextOnOSD(szInput);
        }

        if (fDebug)
            printf("Closing down the server...\n");

        close(serverFd);
        serverFd = 0;
    }

    CleanUp();
    return;
}

void ReadLineFromSocket(int fd, char *szText)
{
    int ch, i;

    // No need to read in anything longer than MAX_CHARS no matter what.
    for (i = 0; i < MAX_CHARS; i++)
    {
        ch = read(fd, szText, 1);
        szText++;

        if (ch == 0)
            goto cleanup;
    }

cleanup:
    szText[i] = '\0';
}

bool DecodeCallerIDInfo(char *szInput, char *szName, char *szNumber)
{
    int ch, i;
    bool fCopyToNumber = false, fIsCallerID = false;

    ch = 0;
    szName[0] = '\0';
    szNumber[0] = '\0';

    fIsCallerID = IsCallerIDInfoAndStripTag(szInput, strlen(szInput));

    if (fIsCallerID)
    {
        // Parse szInput, if applicable, into name and number
        for (i = 0; i < strlen(szInput); i++)
        {
            if ((szInput[i] == '~') && !fCopyToNumber)
            {
                fCopyToNumber = true;
                szName[ch] = '\0';
                ch = -1;
            }
            else if (!fCopyToNumber)
            {
                szName[ch] = szInput[i];
            }
            else
            {
                szNumber[ch] = szInput[i];
            }

            ch++;
        }

        if (fCopyToNumber)
            szNumber[ch] = '\0';

        return true;
    }
    else
        return false;
}

bool IsCallerIDInfoAndStripTag(char *szText, int cbLen)
{
    char szTemp[6], szOutText[MAX_CHARS];
    bool fFoundMarker;
    int i;

    strncpy(szTemp, szText, 5);
    szTemp[5] = '\0';

    if (!strcmp(CALL_TAG, szTemp))
    {
        // This is probably Caller ID info, check for the '~' marker
        fFoundMarker = false;

        for (i = 0; i < strlen(szText); i++)
        {
            if (szText[i] == '~')
            {
                fFoundMarker = true;
                break;
            }
        }

        if (fFoundMarker)
        {
            // Yep, it's definitely Caller ID info, strip tag
            strncpy(szOutText, &szText[5], MAX_CHARS);
            strncpy(szText, szOutText, cbLen);
            return true;
        }
    }

    return false;
}

// Let's do everything we can to try to quit gracefully
void QuitProgram(int sig)
{
    if (fDebug)
        printf("Signal caught; preparing to quit... (signo = %d)\n", sig);
    else
        printf("Please wait... YAC is shutting down.\n");

    fQuit = true;

    if (fListening)
    {
        // We need to take some invasive action of our own.
        // Normally, it would be better to let main() take care of this.

        if (fDebug)
            printf("Currently port listening, quitting from signal code.\n");

        CleanUp();
    }
}

void CleanUp()
{
    // Clean up everything we need to
    if (serverFd)
        close(serverFd);

    if (clientFd)
        close(clientFd);

    if (fDebug)
        printf ("End of the line.  Come debugging again real soon now, ya' hear?\n");
}
